package sun.security.validator;

import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import java.security.NoSuchAlgorithmException;
import java.security.cert.PKIXRevocationChecker;
import java.security.cert.PKIXCertPathChecker;
import java.security.cert.CertPathBuilder;
import java.security.cert.PKIXCertPathBuilderResult;
import java.security.cert.CertStore;
import java.security.cert.CollectionCertStoreParameters;
import java.security.cert.X509CertSelector;
import java.security.GeneralSecurityException;
import java.security.cert.PKIXCertPathValidatorResult;
import java.util.Arrays;
import java.security.cert.CertPathValidator;
import java.util.Date;
import java.security.cert.CertPath;
import java.util.Collections;
import sun.security.provider.certpath.AlgorithmChecker;
import sun.security.provider.certpath.PKIXExtendedParameters;
import java.security.Timestamp;
import java.security.AlgorithmConstraints;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.security.cert.CertificateException;
import java.security.InvalidAlgorithmParameterException;
import java.security.cert.TrustAnchor;
import java.util.HashSet;
import java.util.Collection;
import sun.security.action.GetPropertyAction;
import java.security.cert.CertificateFactory;
import java.security.PublicKey;
import java.util.List;
import javax.security.auth.x500.X500Principal;
import java.util.Map;
import java.security.cert.PKIXBuilderParameters;
import java.security.cert.X509Certificate;
import java.util.Set;

public final class PKIXValidator extends Validator
{
    private static final boolean checkTLSRevocation;
    private static final boolean ALLOW_NON_CA_ANCHOR;
    private final Set<X509Certificate> trustedCerts;
    private final PKIXBuilderParameters parameterTemplate;
    private int certPathLength;
    private final Map<X500Principal, List<PublicKey>> trustedSubjects;
    private final CertificateFactory factory;
    private final boolean plugin;
    
    private static boolean allowNonCaAnchor() {
        String privilegedGetProperty = GetPropertyAction.privilegedGetProperty("jdk.security.allowNonCaAnchor");
        return privilegedGetProperty != null && (privilegedGetProperty.isEmpty() || privilegedGetProperty.equalsIgnoreCase("true"));
    }
    
    PKIXValidator(String defaultParameters, Collection<X509Certificate> c) {
        super("PKIX", defaultParameters);
        this.certPathLength = -1;
        this.trustedCerts = ((c instanceof Set) ? ((Set)c) : new HashSet(c));
        HashSet<TrustAnchor> trustAnchors = new HashSet();
        Iterator<Object> iterator = c.iterator();
        while (iterator.hasNext()) {
            trustAnchors.add(new TrustAnchor((X509Certificate)iterator.next(), null));
        }
        try {
            this.parameterTemplate = new PKIXBuilderParameters(trustAnchors, null);
            this.factory = CertificateFactory.getInstance("X.509");
        }
        catch (InvalidAlgorithmParameterException cause) {
            throw new RuntimeException("Unexpected error: " + cause.toString(), cause);
        }
        catch (CertificateException cause2) {
            throw new RuntimeException("Internal error", cause2);
        }
        this.setDefaultParameters(defaultParameters);
        this.plugin = defaultParameters.equals("plugin code signing");
        this.trustedSubjects = this.setTrustedSubjects();
    }
    
    PKIXValidator(String s, PKIXBuilderParameters parameterTemplate) {
        super("PKIX", s);
        this.certPathLength = -1;
        this.trustedCerts = new HashSet();
        Iterator<TrustAnchor> iterator = parameterTemplate.getTrustAnchors().iterator();
        while (iterator.hasNext()) {
            X509Certificate trustedCert = ((TrustAnchor)iterator.next()).getTrustedCert();
            if (trustedCert != null) {
                this.trustedCerts.add(trustedCert);
            }
            else {
                continue;
            }
        }
        this.parameterTemplate = parameterTemplate;
        try {
            this.factory = CertificateFactory.getInstance("X.509");
        }
        catch (CertificateException cause) {
            throw new RuntimeException("Internal error", cause);
        }
        this.plugin = s.equals("plugin code signing");
        this.trustedSubjects = this.setTrustedSubjects();
    }
    
    private Map<X500Principal, List<PublicKey>> setTrustedSubjects() {
        HashMap<X500Principal, List> hashMap = new HashMap();
        for (X509Certificate x509Certificate : this.trustedCerts) {
            X500Principal subjectX500Principal = x509Certificate.getSubjectX500Principal();
            List<PublicKey> list;
            if (hashMap.containsKey(subjectX500Principal)) {
                list = (Object)hashMap.get(subjectX500Principal);
            }
            else {
                list = new ArrayList();
                hashMap.put(subjectX500Principal, list);
            }
            list.add(x509Certificate.getPublicKey());
        }
        return hashMap;
    }
    
    @Override
    public Collection<X509Certificate> getTrustedCertificates() {
        return this.trustedCerts;
    }
    
    public int getCertPathLength() {
        return this.certPathLength;
    }
    
    private void setDefaultParameters(String s) {
        if (s == "tls server" || s == "tls client") {
            this.parameterTemplate.setRevocationEnabled(PKIXValidator.checkTLSRevocation);
        }
        else {
            this.parameterTemplate.setRevocationEnabled(false);
        }
    }
    
    public PKIXBuilderParameters getParameters() {
        return this.parameterTemplate;
    }
    
    @Override
    X509Certificate[] engineValidate(X509Certificate[] array, Collection<X509Certificate> collection, List<byte[]> list, AlgorithmConstraints algorithmConstraints, Object o) throws CertificateException {
        if (array == null || array.length == 0) {
            throw new CertificateException("null or zero-length certificate chain");
        }
        Object o2 = null;
        try {
            o2 = new PKIXExtendedParameters((PKIXBuilderParameters)this.parameterTemplate.clone(), (o instanceof Timestamp) ? ((Timestamp)o) : null, this.variant);
        }
        catch (InvalidAlgorithmParameterException ex) {}
        if (algorithmConstraints != null) {
            o2.addCertPathChecker(new AlgorithmChecker(algorithmConstraints, null, this.variant));
        }
        if (!list.isEmpty()) {
            addResponses(o2, array, list);
        }
        X500Principal issuerX500Principal = null;
        X509Certificate x509Certificate;
        for (int i = 0; i < array.length; issuerX500Principal = x509Certificate.getIssuerX500Principal(), ++i) {
            x509Certificate = array[i];
            X500Principal subjectX500Principal = x509Certificate.getSubjectX500Principal();
            if (i == 0) {
                if (this.trustedCerts.contains(x509Certificate)) {
                    return new X509Certificate[] { array[0] };
                }
            }
            else {
                if (!subjectX500Principal.equals(issuerX500Principal)) {
                    return this.doBuild(array, collection, o2);
                }
                if (this.trustedCerts.contains(x509Certificate) || (this.trustedSubjects.containsKey(subjectX500Principal) && ((List)this.trustedSubjects.get(subjectX500Principal)).contains(x509Certificate.getPublicKey()))) {
                    X509Certificate[] array2 = new X509Certificate[i];
                    System.arraycopy(array, 0, array2, 0, i);
                    return this.doValidate(array2, o2);
                }
            }
        }
        X509Certificate x509Certificate2 = array[array.length - 1];
        X500Principal issuerX500Principal2 = x509Certificate2.getIssuerX500Principal();
        x509Certificate2.getSubjectX500Principal();
        if (this.trustedSubjects.containsKey(issuerX500Principal2) && this.isSignatureValid((List)this.trustedSubjects.get(issuerX500Principal2), x509Certificate2)) {
            return this.doValidate(array, o2);
        }
        if (this.plugin) {
            if (array.length > 1) {
                X509Certificate[] array3 = new X509Certificate[array.length - 1];
                System.arraycopy(array, 0, array3, 0, array3.length);
                try {
                    o2.setTrustAnchors(Collections.singleton(new TrustAnchor(array[array.length - 1], null)));
                }
                catch (InvalidAlgorithmParameterException cause) {
                    throw new CertificateException(cause);
                }
                this.doValidate(array3, o2);
            }
            throw new ValidatorException(ValidatorException.T_NO_TRUST_ANCHOR);
        }
        else {
            return this.doBuild(array, collection, o2);
        }
    }
    
    private boolean isSignatureValid(List<PublicKey> list, X509Certificate x509Certificate) {
        if (this.plugin) {
            for (PublicKey publicKey : list) {
                try {
                    x509Certificate.verify(publicKey);
                    return true;
                }
                catch (Exception ex) {}
            }
            return false;
        }
        return true;
    }
    
    private static X509Certificate[] toArray(CertPath certPath, TrustAnchor trustAnchor) throws CertificateException {
        X509Certificate trustedCert = trustAnchor.getTrustedCert();
        if (trustedCert == null) {
            throw new ValidatorException("TrustAnchor must be specified as certificate");
        }
        verifyTrustAnchor(trustedCert);
        List<Object> certificates = certPath.getCertificates();
        X509Certificate[] array = new X509Certificate[certificates.size() + 1];
        certificates.toArray(array);
        array[array.length - 1] = trustedCert;
        return array;
    }
    
    private void setDate(PKIXBuilderParameters pkixBuilderParameters) {
        Date validationDate = this.validationDate;
        if (validationDate != null) {
            pkixBuilderParameters.setDate(validationDate);
        }
    }
    
    private X509Certificate[] doValidate(X509Certificate[] a, PKIXBuilderParameters pkixBuilderParameters) throws CertificateException {
        try {
            this.setDate(pkixBuilderParameters);
            CertPathValidator instance = CertPathValidator.getInstance("PKIX");
            CertPath generateCertPath = this.factory.generateCertPath(Arrays.asList(a));
            this.certPathLength = a.length;
            return toArray(generateCertPath, ((PKIXCertPathValidatorResult)instance.validate(generateCertPath, pkixBuilderParameters)).getTrustAnchor());
        }
        catch (GeneralSecurityException ex) {
            throw new ValidatorException("PKIX path validation failed: " + ex.toString(), ex);
        }
    }
    
    private static void verifyTrustAnchor(X509Certificate x509Certificate) throws ValidatorException {
        if (PKIXValidator.ALLOW_NON_CA_ANCHOR) {
            return;
        }
        if (x509Certificate.getVersion() < 3) {
            return;
        }
        if (x509Certificate.getBasicConstraints() == -1) {
            throw new ValidatorException("TrustAnchor with subject \"" + x509Certificate.getSubjectX500Principal() + "\" is not a CA certificate");
        }
        boolean[] keyUsage = x509Certificate.getKeyUsage();
        if (keyUsage != null && !keyUsage[5]) {
            throw new ValidatorException("TrustAnchor with subject \"" + x509Certificate.getSubjectX500Principal() + "\" does not have keyCertSign bit set in KeyUsage extension");
        }
        return;
    }
    
    private X509Certificate[] doBuild(X509Certificate[] a, Collection<X509Certificate> collection, PKIXBuilderParameters pkixBuilderParameters) throws CertificateException {
        try {
            this.setDate(pkixBuilderParameters);
            X509CertSelector targetCertConstraints = new X509CertSelector();
            targetCertConstraints.setCertificate(a[0]);
            pkixBuilderParameters.setTargetCertConstraints(targetCertConstraints);
            ArrayList<Object> collection2 = new ArrayList();
            collection2.addAll(Arrays.asList(a));
            if (collection != null) {
                collection2.addAll(collection);
            }
            pkixBuilderParameters.addCertStore(CertStore.getInstance("Collection", new CollectionCertStoreParameters(collection2)));
            PKIXCertPathBuilderResult pkixCertPathBuilderResult = (PKIXCertPathBuilderResult)CertPathBuilder.getInstance("PKIX").build(pkixBuilderParameters);
            return toArray(pkixCertPathBuilderResult.getCertPath(), pkixCertPathBuilderResult.getTrustAnchor());
        }
        catch (GeneralSecurityException ex) {
            throw new ValidatorException("PKIX path building failed: " + ex.toString(), ex);
        }
    }
    
    private static void addResponses(PKIXBuilderParameters pkixBuilderParameters, X509Certificate[] array, List<byte[]> list) {
        if (pkixBuilderParameters.isRevocationEnabled()) {
            try {
                PKIXRevocationChecker pkixRevocationChecker = null;
                ArrayList certPathCheckers = new ArrayList(pkixBuilderParameters.getCertPathCheckers());
                for (PKIXCertPathChecker pkixCertPathChecker : certPathCheckers) {
                    if ((pkixCertPathChecker instanceof PKIXRevocationChecker)) {
                        pkixRevocationChecker = (PKIXRevocationChecker)pkixCertPathChecker;
                        break;
                    }
                    continue;
                }
                if (pkixRevocationChecker == null) {
                    pkixRevocationChecker = (PKIXRevocationChecker)CertPathValidator.getInstance("PKIX").getRevocationChecker();
                    certPathCheckers.add(pkixRevocationChecker);
                }
                Map<X509Certificate, byte[]> ocspResponses = pkixRevocationChecker.getOcspResponses();
                for (int min = Integer.min(array.length, list.size()), i = 0; i < min; ++i) {
                    byte[] array2 = (byte[])list.get(i);
                    if (array2 != null && array2.length > 0 && !ocspResponses.containsKey(array[i])) {
                        ocspResponses.put(array[i], array2);
                    }
                }
                pkixRevocationChecker.setOcspResponses(ocspResponses);
                pkixBuilderParameters.setCertPathCheckers(certPathCheckers);
            }
            catch (NoSuchAlgorithmException ex) {}
        }
    }
    
    static {
        checkTLSRevocation = (boolean)AccessController.doPrivileged(new GetBooleanAction("com.sun.net.ssl.checkRevocation"));
        ALLOW_NON_CA_ANCHOR = allowNonCaAnchor();
    }
}
